import gzip
import io
from struct import pack, unpack
from Cryptodome import Random


def network_independent_pack_size_to_8bytes(size):
    return pack('>Q', size)  # Q equals unsigned long long


def network_independent_unpack_8bytes_to_size(raw_data):
    return unpack('>Q', raw_data[:8])[0]


# Header for the data before being encrypted and saved off
# [8 bytes rnd][8 bytes digest size][digest variable size][plaintext (variable size)]
def generate_new_header(digest):
    rnd = Random.get_random_bytes(8)
    digest_8byte_size = network_independent_pack_size_to_8bytes(len(digest))
    return rnd + digest_8byte_size + digest


# Returns digest + plaintext data from a decrypted aes file
def extract_digest_and_plaintext_from_data_with_header(decrypted_aes_file):
    compressed_digest_size = decrypted_aes_file[8:8+8]
    digest_size = network_independent_unpack_8bytes_to_size(compressed_digest_size)
    digest = decrypted_aes_file[16:16+digest_size]
    # print('DIGEST SIZE INCOMING:', digest_size)
    return digest, decrypted_aes_file[16+digest_size:]


# Returns a string formatted for human reading, input file_size in bytes
def human_readable_file_size(size_in_bytes, short=False):
    if short:
        suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
    else:
        suffixes = ['Bytes', 'KiloBytes', 'MegaBytes', 'GigaBytes', 'TeraBytes', 'PetaBytes']
    i = 0
    while size_in_bytes >= 1024 and i < len(suffixes)-1:
        size_in_bytes /= 1024.
        i += 1
    f = ('%.2f' % size_in_bytes).rstrip('0').rstrip('.')
    return '%s %s' % (f, suffixes[i])


# Gzip byte data
def g_zip(data: bytes) -> bytes:
    out = io.BytesIO()
    with gzip.GzipFile(fileobj=out, mode='wb') as fo:
        fo.write(data)
    bytes_obj = out.getvalue()
    return bytes_obj


# GUnzip byte data
def g_unzip(bytes_obj: bytes) -> bytes:
    return gzip.decompress(bytes_obj)


# Tries to figure out the public ip address for the network this PC is on
def get_public_ip():
    ip = '127.0.0.1'
    import socket
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # Connect to Mindoki.com and ask for the IP that asked (e.g probably our public ip address)
        s.connect(("www.mindoki.com", 80))
        s.sendall(b"GET /ip_responder.php HTTP/1.1\r\nHost: mindoki.com\r\nAccept: text/html\r\n\r\n")
        r = s.recv(4096).decode()

        split = r.split('MINDOKI_IP_RESPONDER:')
        if len(split) == 2:
            ip = split[1]
    except:
        print("Could not connect to IP responder, please configure your public IP:PORT manually")

    return ip
